# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bayesBLP_rcpp_loop <- function(IV, X, Z, share, J, T, v, R, sigmasqR, A, theta_hat, deltabar, Ad, nu0, s0_sq, VOmega, ssq, cand_cov, theta_bar_initial, r_initial, tau_sq_initial, Omega_initial, delta_initial, tol, keep, nprint) {
    .Call('_bayesm_bayesBLP_rcpp_loop', PACKAGE = 'bayesm', IV, X, Z, share, J, T, v, R, sigmasqR, A, theta_hat, deltabar, Ad, nu0, s0_sq, VOmega, ssq, cand_cov, theta_bar_initial, r_initial, tau_sq_initial, Omega_initial, delta_initial, tol, keep, nprint)
}

breg <- function(y, X, betabar, A) {
    .Call('_bayesm_breg', PACKAGE = 'bayesm', y, X, betabar, A)
}

cgetC <- function(e, k) {
    .Call('_bayesm_cgetC', PACKAGE = 'bayesm', e, k)
}

clusterMix_rcpp_loop <- function(zdraw, cutoff, SILENT, nprint) {
    .Call('_bayesm_clusterMix_rcpp_loop', PACKAGE = 'bayesm', zdraw, cutoff, SILENT, nprint)
}

ghkvec <- function(L, trunpt, above, r, HALTON = TRUE, pn = as.integer( c(0))) {
    .Call('_bayesm_ghkvec', PACKAGE = 'bayesm', L, trunpt, above, r, HALTON, pn)
}

llmnl <- function(beta, y, X) {
    .Call('_bayesm_llmnl', PACKAGE = 'bayesm', beta, y, X)
}

lndIChisq <- function(nu, ssq, X) {
    .Call('_bayesm_lndIChisq', PACKAGE = 'bayesm', nu, ssq, X)
}

lndIWishart <- function(nu, V, IW) {
    .Call('_bayesm_lndIWishart', PACKAGE = 'bayesm', nu, V, IW)
}

lndMvn <- function(x, mu, rooti) {
    .Call('_bayesm_lndMvn', PACKAGE = 'bayesm', x, mu, rooti)
}

lndMvst <- function(x, nu, mu, rooti, NORMC = FALSE) {
    .Call('_bayesm_lndMvst', PACKAGE = 'bayesm', x, nu, mu, rooti, NORMC)
}

rDPGibbs_rcpp_loop <- function(R, keep, nprint, y, lambda_hyper, SCALE, maxuniq, PrioralphaList, gridsize, BayesmConstantA, BayesmConstantnuInc, BayesmConstantDPalpha) {
    .Call('_bayesm_rDPGibbs_rcpp_loop', PACKAGE = 'bayesm', R, keep, nprint, y, lambda_hyper, SCALE, maxuniq, PrioralphaList, gridsize, BayesmConstantA, BayesmConstantnuInc, BayesmConstantDPalpha)
}

rbprobitGibbs_rcpp_loop <- function(y, X, Abetabar, root, beta, sigma, trunpt, above, R, keep, nprint) {
    .Call('_bayesm_rbprobitGibbs_rcpp_loop', PACKAGE = 'bayesm', y, X, Abetabar, root, beta, sigma, trunpt, above, R, keep, nprint)
}

rdirichlet <- function(alpha) {
    .Call('_bayesm_rdirichlet', PACKAGE = 'bayesm', alpha)
}

rhierLinearMixture_rcpp_loop <- function(regdata, Z, deltabar, Ad, mubar, Amu, nu, V, nu_e, ssq, R, keep, nprint, drawdelta, olddelta, a, oldprob, ind, tau) {
    .Call('_bayesm_rhierLinearMixture_rcpp_loop', PACKAGE = 'bayesm', regdata, Z, deltabar, Ad, mubar, Amu, nu, V, nu_e, ssq, R, keep, nprint, drawdelta, olddelta, a, oldprob, ind, tau)
}

rhierLinearModel_rcpp_loop <- function(regdata, Z, Deltabar, A, nu, V, nu_e, ssq, tau, Delta, Vbeta, R, keep, nprint) {
    .Call('_bayesm_rhierLinearModel_rcpp_loop', PACKAGE = 'bayesm', regdata, Z, Deltabar, A, nu, V, nu_e, ssq, tau, Delta, Vbeta, R, keep, nprint)
}

rhierMnlDP_rcpp_loop <- function(R, keep, nprint, lgtdata, Z, deltabar, Ad, PrioralphaList, lambda_hyper, drawdelta, nvar, oldbetas, s, maxuniq, gridsize, BayesmConstantA, BayesmConstantnuInc, BayesmConstantDPalpha) {
    .Call('_bayesm_rhierMnlDP_rcpp_loop', PACKAGE = 'bayesm', R, keep, nprint, lgtdata, Z, deltabar, Ad, PrioralphaList, lambda_hyper, drawdelta, nvar, oldbetas, s, maxuniq, gridsize, BayesmConstantA, BayesmConstantnuInc, BayesmConstantDPalpha)
}

llmnl_con <- function(betastar, y, X, SignRes = as.numeric( c(0))) {
    .Call('_bayesm_llmnl_con', PACKAGE = 'bayesm', betastar, y, X, SignRes)
}

rhierMnlRwMixture_rcpp_loop <- function(lgtdata, Z, deltabar, Ad, mubar, Amu, nu, V, s, R, keep, nprint, drawdelta, olddelta, a, oldprob, oldbetas, ind, SignRes) {
    .Call('_bayesm_rhierMnlRwMixture_rcpp_loop', PACKAGE = 'bayesm', lgtdata, Z, deltabar, Ad, mubar, Amu, nu, V, s, R, keep, nprint, drawdelta, olddelta, a, oldprob, oldbetas, ind, SignRes)
}

rhierNegbinRw_rcpp_loop <- function(regdata, hessdata, Z, Beta, Delta, Deltabar, Adelta, nu, V, a, b, R, keep, sbeta, alphacroot, nprint, rootA, alpha, fixalpha) {
    .Call('_bayesm_rhierNegbinRw_rcpp_loop', PACKAGE = 'bayesm', regdata, hessdata, Z, Beta, Delta, Deltabar, Adelta, nu, V, a, b, R, keep, sbeta, alphacroot, nprint, rootA, alpha, fixalpha)
}

rivDP_rcpp_loop <- function(R, keep, nprint, dimd, mbg, Abg, md, Ad, y, isgamma, z, x, w, delta, PrioralphaList, gridsize, SCALE, maxuniq, scalex, scaley, lambda_hyper, BayesmConstantA, BayesmConstantnu) {
    .Call('_bayesm_rivDP_rcpp_loop', PACKAGE = 'bayesm', R, keep, nprint, dimd, mbg, Abg, md, Ad, y, isgamma, z, x, w, delta, PrioralphaList, gridsize, SCALE, maxuniq, scalex, scaley, lambda_hyper, BayesmConstantA, BayesmConstantnu)
}

rivGibbs_rcpp_loop <- function(y, x, z, w, mbg, Abg, md, Ad, V, nu, R, keep, nprint) {
    .Call('_bayesm_rivGibbs_rcpp_loop', PACKAGE = 'bayesm', y, x, z, w, mbg, Abg, md, Ad, V, nu, R, keep, nprint)
}

rmixGibbs <- function(y, Bbar, A, nu, V, a, p, z) {
    .Call('_bayesm_rmixGibbs', PACKAGE = 'bayesm', y, Bbar, A, nu, V, a, p, z)
}

rmixture <- function(n, pvec, comps) {
    .Call('_bayesm_rmixture', PACKAGE = 'bayesm', n, pvec, comps)
}

rmnlIndepMetrop_rcpp_loop <- function(R, keep, nu, betastar, root, y, X, betabar, rootpi, rooti, oldlimp, oldlpost, nprint) {
    .Call('_bayesm_rmnlIndepMetrop_rcpp_loop', PACKAGE = 'bayesm', R, keep, nu, betastar, root, y, X, betabar, rootpi, rooti, oldlimp, oldlpost, nprint)
}

rmnpGibbs_rcpp_loop <- function(R, keep, nprint, pm1, y, X, beta0, sigma0, V, nu, betabar, A) {
    .Call('_bayesm_rmnpGibbs_rcpp_loop', PACKAGE = 'bayesm', R, keep, nprint, pm1, y, X, beta0, sigma0, V, nu, betabar, A)
}

rmultireg <- function(Y, X, Bbar, A, nu, V) {
    .Call('_bayesm_rmultireg', PACKAGE = 'bayesm', Y, X, Bbar, A, nu, V)
}

rmvpGibbs_rcpp_loop <- function(R, keep, nprint, p, y, X, beta0, sigma0, V, nu, betabar, A) {
    .Call('_bayesm_rmvpGibbs_rcpp_loop', PACKAGE = 'bayesm', R, keep, nprint, p, y, X, beta0, sigma0, V, nu, betabar, A)
}

rmvst <- function(nu, mu, root) {
    .Call('_bayesm_rmvst', PACKAGE = 'bayesm', nu, mu, root)
}

rnegbinRw_rcpp_loop <- function(y, X, betabar, rootA, a, b, beta, alpha, fixalpha, betaroot, alphacroot, R, keep, nprint) {
    .Call('_bayesm_rnegbinRw_rcpp_loop', PACKAGE = 'bayesm', y, X, betabar, rootA, a, b, beta, alpha, fixalpha, betaroot, alphacroot, R, keep, nprint)
}

rnmixGibbs_rcpp_loop <- function(y, Mubar, A, nu, V, a, p, z, R, keep, nprint) {
    .Call('_bayesm_rnmixGibbs_rcpp_loop', PACKAGE = 'bayesm', y, Mubar, A, nu, V, a, p, z, R, keep, nprint)
}

rordprobitGibbs_rcpp_loop <- function(y, X, k, A, betabar, Ad, s, inc_root, dstarbar, betahat, R, keep, nprint) {
    .Call('_bayesm_rordprobitGibbs_rcpp_loop', PACKAGE = 'bayesm', y, X, k, A, betabar, Ad, s, inc_root, dstarbar, betahat, R, keep, nprint)
}

rscaleUsage_rcpp_loop <- function(k, x, p, n, R, keep, ndghk, nprint, y, mu, Sigma, tau, sigma, Lambda, e, domu, doSigma, dosigma, dotau, doLambda, doe, nu, V, mubar, Am, gsigma, gl11, gl22, gl12, nuL, VL, ge) {
    .Call('_bayesm_rscaleUsage_rcpp_loop', PACKAGE = 'bayesm', k, x, p, n, R, keep, ndghk, nprint, y, mu, Sigma, tau, sigma, Lambda, e, domu, doSigma, dosigma, dotau, doLambda, doe, nu, V, mubar, Am, gsigma, gl11, gl22, gl12, nuL, VL, ge)
}

rsurGibbs_rcpp_loop <- function(regdata, indreg, cumnk, nk, XspXs, Sigmainv, A, Abetabar, nu, V, nvar, E, Y, R, keep, nprint) {
    .Call('_bayesm_rsurGibbs_rcpp_loop', PACKAGE = 'bayesm', regdata, indreg, cumnk, nk, XspXs, Sigmainv, A, Abetabar, nu, V, nvar, E, Y, R, keep, nprint)
}

rtrun <- function(mu, sigma, a, b) {
    .Call('_bayesm_rtrun', PACKAGE = 'bayesm', mu, sigma, a, b)
}

runiregGibbs_rcpp_loop <- function(y, X, betabar, A, nu, ssq, sigmasq, R, keep, nprint) {
    .Call('_bayesm_runiregGibbs_rcpp_loop', PACKAGE = 'bayesm', y, X, betabar, A, nu, ssq, sigmasq, R, keep, nprint)
}

runireg_rcpp_loop <- function(y, X, betabar, A, nu, ssq, R, keep, nprint) {
    .Call('_bayesm_runireg_rcpp_loop', PACKAGE = 'bayesm', y, X, betabar, A, nu, ssq, R, keep, nprint)
}

rwishart <- function(nu, V) {
    .Call('_bayesm_rwishart', PACKAGE = 'bayesm', nu, V)
}

callroot <- function(c1, c2, tol, iterlim) {
    .Call('_bayesm_callroot', PACKAGE = 'bayesm', c1, c2, tol, iterlim)
}

